﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using System.Diagnostics;
using Unitilities;

public class Mira : MonoBehaviour {

	public bool bUsePooling = false;

	// This is the bullet that you will be pew pewing at enemies
	public GameObject bulletPrefab;

	public float movementSpeed;
	public float jumpSpeed;
	public float shootingSpeed;

	public float shootingCooldown;

	// This is a custom movement script I made to not use the character controller, please pay it no mind.
	private Movement _movementScript;
	private SpriteRenderer _spriteRenderer;
	private Animator _animator;

	private Facing _facing = Facing.Right;

	private float _shootTimer;

	private Stopwatch bulletStopwatch;

	void Awake() {
		_movementScript = GetComponent<Movement>();
		_spriteRenderer = GetComponentInChildren<SpriteRenderer>();
		_animator = GetComponent<Animator>();
	}

	// Use this for initialization
	void Start () {
		bulletStopwatch = new Stopwatch();

		_movementScript.groundedCallback = Landed;

		if (bUsePooling) {
			GameObjectPool.Warm(bulletPrefab, 20);
		}
	}

	/// <summary>
	/// I just like to sepparate the input and what is being controller by it.
	/// </summary>
	public void UpdateMovement(Facing facing) {
		switch (facing) {
			case Facing.None:
				_movementScript.currentMovement = Vector2.zero;
				break;
			case Facing.Right:
				_facing = facing;
				_spriteRenderer.flipX = false;
				_movementScript.currentMovement = Vector2.right * movementSpeed;
				break;
			case Facing.Left:
				_facing = facing;
				_spriteRenderer.flipX = true;
				_movementScript.currentMovement = Vector2.left * movementSpeed;
				break;
		}
	}

	public void Jump() {
		if (!_movementScript.IsGrounded) {
			return;
		}

		_animator.SetBool("Jumping", true);

		_movementScript.Jump();
		_movementScript.ReceiveKnockback(Vector2.up * jumpSpeed, Time.deltaTime * 20f);
	}

	void Landed() {
		_animator.SetBool("Jumping", false);
	}

	public void Shoot() {
		if (Time.timeSinceLevelLoad < _shootTimer) {
			return;
		}

		bulletStopwatch.Start();

		_animator.SetTrigger("Shoot");

		_shootTimer = Time.timeSinceLevelLoad + shootingCooldown;

		GameObject newBullet = null;
		Bullet bulletScript = null;

		if (bUsePooling) {
			newBullet = GameObjectPool.Get(bulletPrefab);
			newBullet.transform.position = transform.position + Vector3.up * 7f;
		} else {
			newBullet = Instantiate(bulletPrefab, transform.position + Vector3.up * 7f, Quaternion.identity);
		}

		bulletScript = newBullet.GetComponent<Bullet>();

		bulletScript.Shoot(_facing, shootingSpeed, bUsePooling);

		bulletStopwatch.Stop();
		Performance.instance.ShootingPerformance(bulletStopwatch.ElapsedTicks);
		bulletStopwatch.Reset();
	}
}
